'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.draft = draft;
exports.publish = publish;
exports.unpublish = unpublish;
exports.reject = reject;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _ = require('../../');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function draft(filePath, tplPath, json) {
  var workflow = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'draft';
  var type = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 'draft';

  var p = new Promise(function (resolve, reject) {
    _.abeExtend.hooks.instance.trigger('beforeDraft', json, filePath, tplPath);
    _.cmsOperations.save.save(_path2.default.join(_.config.root, _.config.draft.url, filePath.replace(_.config.root)), tplPath, json, '', workflow, null, type).then(function (resSave) {
      var result;
      if (typeof resSave.error !== 'undefined' && resSave.error !== null) {
        result = { error: resSave.error };
      } else if (typeof resSave.reject !== 'undefined' && resSave.reject !== null) {
        result = resSave;
      } else if (typeof resSave.json !== 'undefined' && resSave.json !== null) {
        _.Manager.instance.updatePostInList(resSave.jsonPath);
        result = {
          success: 1,
          json: resSave.json
        };
      }
      resolve(result);
    });
  });

  return p;
}

function publish(filePath, tplPath, json) {
  var p = new Promise(function (resolve, reject) {
    _.abeExtend.hooks.instance.trigger('beforePublish', json, filePath, tplPath);
    var p1 = new Promise(function (resolve) {
      _.cmsOperations.save.save(_path2.default.join(_.config.root, _.config.draft.url, filePath.replace(_.config.root)), tplPath, json, '', 'draft', null, 'publish').then(function () {
        resolve();
      }).catch(function (e) {
        console.error(e);
      });
    });

    p1.then(function (resSave) {
      _.cmsOperations.save.save(_path2.default.join(_.config.root, _.config.draft.url, filePath.replace(_.config.root)), tplPath, json, '', 'publish', resSave, 'publish').then(function (resSave) {
        var result;
        if (typeof resSave.error !== 'undefined' && resSave.error !== null) {
          result = {
            success: 0,
            error: resSave.error
          };
        } else if (typeof resSave.reject !== 'undefined' && resSave.reject !== null) {
          result = resSave;
        } else if (typeof resSave.json !== 'undefined' && resSave.json !== null) {
          result = {
            success: 1,
            json: resSave.json
          };
        }
        _.abeExtend.hooks.instance.trigger('afterPublish', result);
        _.Manager.instance.updatePostInList(resSave.jsonPath);
        resolve(result);
      }).catch(function (e) {
        console.error('post.js', e);
        var result = {
          success: 0,
          error: 'publish error'
        };
        _.abeExtend.hooks.instance.trigger('afterPublish', result);
        resolve(result);
      });
    }).catch(function (e) {
      console.error('post.js', e);
      var result = {
        success: 0,
        error: 'publish error'
      };
      _.abeExtend.hooks.instance.trigger('afterPublish', result);
      resolve(result);
    });
  });

  return p;
}

function unpublish(filePath) {
  _.abeExtend.hooks.instance.trigger('beforeUnpublish', filePath);
  filePath = _.coreUtils.slug.clean(filePath);
  var tplUrl = _.cmsData.file.fromUrl(_path2.default.join(_.config.publish.url, filePath));
  if (_.coreUtils.file.exist(tplUrl.json.path)) {
    var json = JSON.parse(JSON.stringify(_.cmsData.file.get(tplUrl.json.path)));
    if (json.abe_meta.publish != null) {
      delete json.abe_meta.publish;
    }

    _.cmsOperations.save.save(_path2.default.join(_.config.root, _.config.draft.url, json.abe_meta.link.replace(_.config.root)), json.abe_meta.template, json, '', 'reject', null, 'reject').then(function (resSave) {
      _.cmsOperations.remove.removeFile(tplUrl.publish.path, tplUrl.publish.json);
      _.abeExtend.hooks.instance.trigger('afterUnpublish', tplUrl.publish.path, tplUrl.publish.json);
      _.Manager.instance.updatePostInList(resSave.jsonPath);
    });
  }
}

function reject(filePath, tplPath, json) {
  _.abeExtend.hooks.instance.trigger('beforeReject', filePath);
  var p = new Promise(function (resolve, reject) {
    var p1 = new Promise(function (resolve) {
      _.cmsOperations.save.save(_path2.default.join(_.config.root, _.config.draft.url, filePath.replace(_.config.root)), tplPath, json, '', 'draft', null, 'reject').then(function (resSave) {
        resolve();
      }).catch(function (e) {
        console.error(e);
      });
    });

    p1.then(function (resSave) {
      _.cmsOperations.save.save(_path2.default.join(_.config.root, _.config.draft.url, filePath.replace(_.config.root)), tplPath, json, '', 'reject', resSave, 'reject').then(function (resSave) {
        var result;
        if (typeof resSave.error !== 'undefined' && resSave.error !== null) {
          result = {
            success: 0,
            error: resSave.error
          };
        } else if (typeof resSave.reject !== 'undefined' && resSave.reject !== null) {
          resSave.success = 1;
          result = resSave;
        } else if (typeof resSave.json !== 'undefined' && resSave.json !== null) {
          result = {
            success: 1,
            json: resSave.json
          };
        }
        _.abeExtend.hooks.instance.trigger('afterReject', result);
        _.Manager.instance.updatePostInList(resSave.jsonPath);
        resolve(result);
      });
    }).catch(function (e) {
      console.error(e);
      var result = {
        success: 0,
        error: 'reject error'
      };
      _.abeExtend.hooks.instance.trigger('afterReject', result);
      resolve(result);
    });
  });

  return p;
}