'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _xss = require('xss');

var _xss2 = _interopRequireDefault(_xss);

var _package = require('../../../package');

var _package2 = _interopRequireDefault(_package);

var _cli = require('../../cli');

var _editor = require('../controllers/editor');

var _abeLocale = require('../helpers/abe-locale');

var _abeLocale2 = _interopRequireDefault(_abeLocale);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var route = function route(req, res, next) {
  var filePath = req.originalUrl.replace('/abe', '');
  if (filePath === '' || filePath === '/') {
    filePath = null;
  }

  if (filePath != null) {
    var testXSS = (0, _xss2.default)(filePath, {
      whiteList: [],
      stripIgnoreTag: true
    });
    if (testXSS !== filePath) {
      filePath = testXSS;
    }
  }

  _cli.abeExtend.hooks.instance.trigger('beforeRoute', req, res, next);
  if (typeof res._header !== 'undefined' && res._header !== null) return;

  var isHome = true;
  var jsonPath = null;
  var linkPath = null;
  var template = null;
  var fileName = null;
  var folderPath = null;

  var p = new Promise(function (resolve) {

    if (filePath != null) {
      fileName = filePath.split('/');
      fileName = fileName[fileName.length - 1].replace('.' + _cli.config.files.templates.extension, '');

      folderPath = filePath.split('/');
      folderPath.pop();
      folderPath = folderPath.join('/');

      isHome = false;

      var filePathTest = _cli.cmsData.revision.getDocumentRevision(filePath);
      if (typeof filePathTest !== 'undefined' && filePathTest !== null) {
        jsonPath = filePathTest.path;
        linkPath = filePathTest.abe_meta.link;
        template = filePathTest.abe_meta.template;
      }

      if (jsonPath === null || !_cli.coreUtils.file.exist(jsonPath)) {
        res.redirect('/abe/');
        return;
      }

      (0, _editor.editor)(template, jsonPath, linkPath).then(function (result) {
        resolve(result);
      }).catch(function (e) {
        console.error(e);
      });
    } else {
      resolve({
        obj: {},
        manager: {}
      });
    }
  }).catch(function (e) {
    console.error(e); // "oh, no!"
  });

  p.then(function (result) {
    var obj = result;
    var manager = {};

    manager.home = {
      files: _cli.Manager.instance.getList()
    };

    manager.list = _cli.Manager.instance.getStructureAndTemplates();
    manager.editConfig = req.app.get('config');
    manager.config = JSON.stringify(_cli.config);

    var _hasBlock = obj ? obj.hasBlock : false;
    var _hasSingleBlock = obj ? obj.hasSingleBlock : false;
    var _preview = filePath ? '/abe/page/' + req.params[0] + ('?filePath=' + req.query.filePath) : false;
    var _form = obj ? obj.form : false;
    var _json = obj ? obj.json : false;
    var _text = obj ? obj.text : false;
    // var _file = (tplUrl) ? tplUrl.draft.file : false
    var _filePath = filePath ? filePath : false;
    if (_filePath) {
      _filePath = '/' + _filePath.replace(/^\/+/, '');
    }

    var pageHtml = '';
    if (typeof _json !== 'undefined' && _json !== null && typeof _json.abe_meta !== 'undefined' && _json.abe_meta !== null) {

      var text = _cli.cmsTemplates.template.getTemplate(_json.abe_meta.template);
      var page = new _cli.Page(_json.abe_meta.template, text, _json, false);
      pageHtml = page.html.replace(/"/g, '"').replace(/'/g, '\'').replace(/<!--/g, '<ABE!--').replace(/-->/g, '--ABE>');
    }

    var EditorVariables = {
      pageHtml: pageHtml,
      isHome: isHome,
      abeUrl: '/abe/',
      test: JSON.stringify(_abeLocale2.default),
      text: _abeLocale2.default,
      preview: _preview,
      filename: fileName,
      folderPath: folderPath,
      hasSingleBlock: _hasSingleBlock,
      hasBlock: _hasBlock,
      form: _form,
      json: _json,
      config: _cli.config,
      Locales: _cli.coreUtils.locales.instance.i18n,
      manager: manager,
      express: {
        res: res,
        req: req
      },
      abeVersion: _package2.default.version,
      nonce: '\'nonce-' + res.locals.nonce + '\''
    };
    EditorVariables = _cli.abeExtend.hooks.instance.trigger('afterVariables', EditorVariables);

    if (filePath != null && filePath.indexOf('.json') > -1) {
      res.set('Content-Type', 'application/json');
      res.send(JSON.stringify(_json));
    } else {
      res.render(_cli.config.abeEngine, EditorVariables);
    }
  }).catch(function (e) {
    console.log('error', e);
  });
};

exports.default = route;